//
// Created by hbini on 4/18/24.
//

#ifndef ACL_RESNET50_UTILS_OUTPUT_H
#define ACL_RESNET50_UTILS_OUTPUT_H

#include "fcos_parameters.h"

template<typename T>
struct ClsScores {
    static constexpr size_t len = 5;
    Vec<Tensor_R<T, 3>> scrs;

    explicit ClsScores(const Fcos3dConstantParameter fcos3DConstantParameter) {
        auto num_cls = fcos3DConstantParameter.num_cls;
        scrs = Vec<Tensor_R<T, 3>>{{num_cls, fcos3DConstantParameter.fpn0_size[0], fcos3DConstantParameter.fpn0_size[1]},
                                   {num_cls, fcos3DConstantParameter.fpn1_size[0], fcos3DConstantParameter.fpn1_size[1]},
                                   {num_cls, fcos3DConstantParameter.fpn2_size[0], fcos3DConstantParameter.fpn2_size[1]},
                                   {num_cls, fcos3DConstantParameter.fpn3_size[0], fcos3DConstantParameter.fpn3_size[1]},
                                   {num_cls, fcos3DConstantParameter.fpn4_size[0], fcos3DConstantParameter.fpn4_size[1]}};
    }

};

template<typename T>
struct BboxPreds {
    static constexpr size_t len = 5;
    Vec<Tensor_R<T, 3>> bboxes;

    explicit BboxPreds(const Fcos3dConstantParameter fcos3DConstantParameter) {
        auto num_bbox = fcos3DConstantParameter.num_bbox;
        bboxes = Vec<Tensor_R<T, 3>>{{num_bbox, fcos3DConstantParameter.fpn0_size[0], fcos3DConstantParameter.fpn0_size[1]},
                                     {num_bbox, fcos3DConstantParameter.fpn1_size[0], fcos3DConstantParameter.fpn1_size[1]},
                                     {num_bbox, fcos3DConstantParameter.fpn2_size[0], fcos3DConstantParameter.fpn2_size[1]},
                                     {num_bbox, fcos3DConstantParameter.fpn3_size[0], fcos3DConstantParameter.fpn3_size[1]},
                                     {num_bbox, fcos3DConstantParameter.fpn4_size[0], fcos3DConstantParameter.fpn4_size[1]}};
    }


};

template<typename T>
struct DirClsPreds {
    static constexpr size_t len = 5;
    Vec<Tensor_R<T, 3>> dirs;

    explicit DirClsPreds(const Fcos3dConstantParameter fcos3DConstantParameter) {
        auto num_dir_cls = fcos3DConstantParameter.num_dir_cls;
        dirs = Vec<Tensor_R<T, 3>>{{num_dir_cls, fcos3DConstantParameter.fpn0_size[0], fcos3DConstantParameter.fpn0_size[1]},
                                   {num_dir_cls, fcos3DConstantParameter.fpn1_size[0], fcos3DConstantParameter.fpn1_size[1]},
                                   {num_dir_cls, fcos3DConstantParameter.fpn2_size[0], fcos3DConstantParameter.fpn2_size[1]},
                                   {num_dir_cls, fcos3DConstantParameter.fpn3_size[0], fcos3DConstantParameter.fpn3_size[1]},
                                   {num_dir_cls, fcos3DConstantParameter.fpn4_size[0], fcos3DConstantParameter.fpn4_size[1]}};
    }


};

template<typename T>
struct AttrPreds {
    static constexpr size_t len = 5;
    Vec<Tensor_R<T, 3>> attr;

    explicit AttrPreds(const Fcos3dConstantParameter fcos3DConstantParameter) {
        auto num_arrt = fcos3DConstantParameter.num_arrt;
        attr = Vec<Tensor_R<T, 3>>{{num_arrt, fcos3DConstantParameter.fpn0_size[0], fcos3DConstantParameter.fpn0_size[1]},
                                   {num_arrt, fcos3DConstantParameter.fpn1_size[0], fcos3DConstantParameter.fpn1_size[1]},
                                   {num_arrt, fcos3DConstantParameter.fpn2_size[0], fcos3DConstantParameter.fpn2_size[1]},
                                   {num_arrt, fcos3DConstantParameter.fpn3_size[0], fcos3DConstantParameter.fpn3_size[1]},
                                   {num_arrt, fcos3DConstantParameter.fpn4_size[0], fcos3DConstantParameter.fpn4_size[1]}};
    }


};

template<typename T>
struct Centerness {
    static constexpr size_t len = 5;
    Vec<Tensor_R<T, 3>> centers;

    explicit Centerness(const Fcos3dConstantParameter fcos3DConstantParameter) {
        auto num_centerness = fcos3DConstantParameter.num_centerness;
        centers = Vec<Tensor_R<T, 3>>{{num_centerness, fcos3DConstantParameter.fpn0_size[0], fcos3DConstantParameter.fpn0_size[1]},
                                      {num_centerness, fcos3DConstantParameter.fpn1_size[0], fcos3DConstantParameter.fpn1_size[1]},
                                      {num_centerness, fcos3DConstantParameter.fpn2_size[0], fcos3DConstantParameter.fpn2_size[1]},
                                      {num_centerness, fcos3DConstantParameter.fpn3_size[0], fcos3DConstantParameter.fpn3_size[1]},
                                      {num_centerness, fcos3DConstantParameter.fpn4_size[0], fcos3DConstantParameter.fpn4_size[1]}};
    }
};


#endif //ACL_RESNET50_UTILS_OUTPUT_H
